@@ -6,16 +6,17 @@ from nonebot.adapters.onebot.v11 import Adapter
 from nonebot.drivers import ReverseDriver
 from nonebot.log import default_filter, default_format
 
-from . import plugin, web  # noqa: F401
-from .log import LOG_STORAGE, logger
-from .plugin_config import config
-from .process import (
+from nonebot_plugin_gocqhttp import web  # noqa: F401
+from nonebot_plugin_gocqhttp.log import LOG_STORAGE, logger
+from nonebot_plugin_gocqhttp.plugin_config import config
+from nonebot_plugin_gocqhttp.process import (
     ACCOUNTS_SAVE_PATH,
     BINARY_PATH,
     ProcessesManager,
     download_gocq,
     kill_duplicated_processes,
 )
+from nonebot_plugin_gocqhttp.proxy import ProxyServiceManager
 
 driver = get_driver()
 
@@ -32,9 +33,7 @@ async def startup():
     loop = asyncio.get_running_loop()
 
     def log_sink(message: str):
-        asyncio.run_coroutine_threadsafe(
-            LOG_STORAGE.add(message.rstrip("\n")), loop=loop
-        )
+        loop.create_task(LOG_STORAGE.add(message.rstrip("\n")))
 
     logger.add(log_sink, colorize=True, filter=default_filter, format=default_format)
 
@@ -52,21 +51,22 @@ async def startup():
         return_exceptions=True,
     )
 
-    if web.DIST_PATH.is_dir():
-        logger.info(
-            "Startup complete, Web UI has served to "
-            f"<u><e>http://127.0.0.1:{driver.config.port}/go-cqhttp/</e></u>"
-        )
+    if tunnel_port := config.TUNNEL_PORT:
+        try:
+            import proxy  # noqa:F401
+        except ImportError as e:
+            logger.warning(f"Tunnel configured but required dependencies missing: {e}")
+        await ProxyServiceManager.start(tunnel_port)
 
-    return
+    logger.info(
+        "Startup complete, Web UI has served to "
+        f"<u><e>http://{driver.config.host}:{driver.config.port}/go-cqhttp/</e></u>"
+    )
 
 
 @driver.on_shutdown
 async def shutdown():
-    await ProcessesManager.save()
-
     await asyncio.gather(
         *map(lambda process: process.stop(), ProcessesManager.all()),
         return_exceptions=True,
     )
-    return
